/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EnvEntriesExtensionListener.java 4310 2008-11-13 09:50:55Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.helper.listener;

import org.ow2.easybeans.event.naming.EnvNamingEvent;
import org.ow2.util.ee.metadata.common.api.struct.IEnvEntry;
import org.ow2.util.event.api.IEvent;

import javax.naming.Context;
import javax.naming.NamingException;

/**
 * This ExtensionListener is dedicated to build env entries into
 * the <code>java:comp/env</code> Context.
 * @author Guillaume Sauthier
 */
public class EnvEntriesExtensionListener extends AbstractExtensionListener {

    /**
     * Only accepts the event if it's an EnvNamingEvent.
     * @param event tested event
     * @return <code>true</code> if the proposed event is of the expected type
     */
    @Override
    public boolean accept(final IEvent event) {
        if (event instanceof EnvNamingEvent) {
            EnvNamingEvent namingEvent = (EnvNamingEvent) event;

            // source/event-provider-id attribute is used to filter the destination
            if ("comp/env".equals(namingEvent.getEventProviderId())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Process the event.
     * @param event
     */
    public void handle(final IEvent event) {

        EnvNamingEvent ene = (EnvNamingEvent) event;
        Context envCtx = ene.getContext();

        for (IEnvEntry envEntry : ene.getBeanMetadata().getEnvEntryCollection()) {
            String name = envEntry.getName();
            Object value = getEnvEntryValue(envEntry, ene);
            // if null, no entry in java:comp/env as specified chapter 15.4.1
            if (value != null) {
                try {
                    envCtx.rebind(name, value);
                } catch (NamingException e) {
                    throwException(ene, new IllegalStateException("Cannot bind element '" + name + "'.", e));
                }
            }
        }
    }

    /**
     * Gets the value for a given env-entry.
     * @param envEntry the element representing env-entry.
     * @param event the processed event
     * @return the value associated to the given element.
     */
    private Object getEnvEntryValue(final IEnvEntry envEntry, EnvNamingEvent event) {
        final String type = envEntry.getType();
        final String value = envEntry.getValue();

        Object returnedValue = null;

        if (type.equals(Boolean.class.getName())) {
            if ("true".equalsIgnoreCase(value)) {
                returnedValue = Boolean.TRUE;
            } else if ("false".equalsIgnoreCase(value)) {
                returnedValue = Boolean.FALSE;
            }
        } else if (type.equals(String.class.getName())) {
            returnedValue = value;
        } else if (type.equals(Integer.class.getName())) {
            if (value != null) {
                returnedValue = new Integer(value);
            }
        } else if (type.equals(Character.class.getName())) {
            if (value != null) {
                if (value.length() != 1) {
                    throwException(event, new IllegalStateException("The value '" + value
                                          + "' is not a valid value for env-entry of type java.lang.Character."));
                }
                returnedValue = Character.valueOf(value.charAt(0));
            }
        } else if (type.equals(Double.class.getName())) {
            if (value != null) {
                returnedValue = new Double(value);
            }
        } else if (type.equals(Byte.class.getName())) {
            if (value != null) {
                returnedValue = new Byte(value);
            }
        } else if (type.equals(Short.class.getName())) {
            if (value != null) {
                returnedValue = new Short(value);
            }
        } else if (type.equals(Long.class.getName())) {
            if (value != null) {
                returnedValue = new Long(value);
            }
        } else if (type.equals(Float.class.getName())) {
            if (value != null) {
                returnedValue = new Float(value);
            }
        } else {
            throwException(event, new IllegalStateException(type + " is not a valid type for env-entry."));
        }
        return returnedValue;
    }
}
